@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2021 - 2023                                             *
+ *   Copyright (C) 2021 - 2025                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -18,15 +18,15 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
+#include "campaign_data.h"
+
 #include <algorithm>
 #include <array>
 #include <cassert>
 #include <cstddef>
-#include <memory>
 #include <utility>
 
 #include "artifact.h"
-#include "campaign_data.h"
 #include "color.h"
 #include "game_video_type.h"
 #include "heroes.h"
@@ -185,7 +185,7 @@ namespace
                 "The local lords refuse to swear allegiance to Roland, and must be subdued. They are wealthy and powerful, so be prepared for a tough fight. Capture all enemy castles to win." ),
             gettext_noop(
                 "Your task is to defend the Dwarves against Archibald's forces. Capture all of the enemy towns and castles to win, and be sure not to lose all of the dwarf towns at once, or the enemy will have won." ),
-            gettext_noop( "You will face four allied enemies in a straightforward fight for resource and treasure. Capture all of the enemy castles for victory." ),
+            gettext_noop( "You will face four allied enemies in a straightforward fight for resources and treasure. Capture all of the enemy castles for victory." ),
             gettext_noop(
                 "Your enemies are allied against you and start close by, so be ready to come out fighting. You will need to own all four castles in this small valley to win." ),
             gettext_noop(
@@ -444,12 +444,12 @@ namespace
             gettext_noop( "Your rival, the Kingdom of Harondale, is attacking weak towns on your border! Recover from their first strike and crush them completely!" ),
             gettext_noop(
                 "Find your wayward son Joseph who is rumored to be living in the desolate lands. Do it before the first day of the third month or it will be of no help to your family." ),
-            gettext_noop( "Rescue your crazy uncle Ivan. Find him before the first day of the fourth month or it will be no help to your kingdom." ),
+            gettext_noop( "Rescue your crazy uncle Ivan. Find him before the first day of the fourth month or it will be of no help to your kingdom." ),
             gettext_noop(
                 "Destroy the barbarians who are attacking the southern border of your kingdom! Recover your fallen towns, and then invade the jungle kingdom. Leave no enemy standing." ),
             gettext_noop( "Retake the castle of Ivory Gates, which has fallen due to treachery." ),
             gettext_noop(
-                "Gain the favor of the elves. They will not allow trees to be chopped down, so they will send you wood every 2 weeks. You must complete your mission before the first day of the seventh month, or the kingdom will surely fall." ),
+                "Gain the favor of the elves. They will not allow trees to be chopped down, so we will send you wood every 2 weeks. You must complete your mission before the first day of the seventh month, or the kingdom will surely fall." ),
             gettext_noop( "This is the final battle against your rival kingdom of Harondale. Eliminate everyone, and don't lose the hero Jarkonas VI." ) };
         std::vector<Campaign::ScenarioData> scenarioDatas;
         scenarioDatas.reserve( 8 );
@@ -609,11 +609,14 @@ namespace Campaign
         case DESCENDANTS_CAMPAIGN:
         case WIZARDS_ISLE_CAMPAIGN:
         case VOYAGE_HOME_CAMPAIGN:
+            break;
         default:
+            // Did you add a new campaign? Add the case handling code for it!
+            assert( 0 );
             break;
         }
 
-        return std::vector<Campaign::CampaignAwardData>();
+        return {};
     }
 
     const std::vector<ScenarioInfoId> & CampaignData::getScenariosAfter( const ScenarioInfoId & scenarioInfo )
@@ -741,7 +744,7 @@ namespace Campaign
         }
 
         if ( allAIPlayersInAlliance ) {
-            const Colors humanColors( mapInfo.colorsAvailableForHumans );
+            const PlayerColorsVector humanColors( mapInfo.colorsAvailableForHumans );
             // Make sure that this is only one human player on the map.
             if ( humanColors.size() != 1 ) {
                 // Looks like somebody is modifying the original map.
@@ -749,21 +752,18 @@ namespace Campaign
                 return;
             }
 
-            const int aiColors = ( mapInfo.kingdomColors & ( ~mapInfo.colorsAvailableForHumans ) );
+            const PlayerColorsSet aiColors = ( mapInfo.kingdomColors & ( ~mapInfo.colorsAvailableForHumans ) );
             if ( aiColors == 0 ) {
                 // This is definitely not the map to modify.
                 assert( 0 );
                 return;
             }
 
-            // If this assertion blows up then the whole logic behind colors is going crazy.
-            assert( aiColors < 256 );
+            const PlayerColor humanColor = humanColors.front();
 
-            const uint8_t humanColor = static_cast<uint8_t>( humanColors.front() );
-
-            for ( uint8_t & allianceColor : mapInfo.unions ) {
-                if ( allianceColor != humanColor && ( aiColors & allianceColor ) ) {
-                    allianceColor = static_cast<uint8_t>( aiColors );
+            for ( PlayerColorsSet & allianceColor : mapInfo.unions ) {
+                if ( allianceColor != static_cast<PlayerColorsSet>( humanColor ) && ( allianceColor & aiColors ) ) {
+                    allianceColor = aiColors;
                 }
             }
         }
@@ -863,12 +863,12 @@ namespace Campaign
             return description;
         }
         case CampaignAwardData::TYPE_GET_ARTIFACT: {
-            std::string description( _( "\"%{artifact}\" artifact will be carried over the scenario." ) );
+            std::string description( _( "\"%{artifact}\" artifact will be carried over in the campaign." ) );
             StringReplace( description, "%{artifact}", Artifact( _subType ).GetName() );
             return description;
         }
         case CampaignAwardData::TYPE_CARRY_OVER_FORCES: {
-            return _( "The army will be carried over the scenario." );
+            return _( "The army will be carried over in the campaign." );
         }
         case CampaignAwardData::TYPE_RESOURCE_BONUS: {
             std::string description( _( "The kingdom will have +%{count} %{resource} each day." ) );
@@ -877,17 +877,17 @@ namespace Campaign
             return description;
         }
         case CampaignAwardData::TYPE_GET_SPELL: {
-            std::string description( _( "\"%{spell}\" spell will be carried over the scenario." ) );
+            std::string description( _( "The \"%{spell}\" spell will be carried over in the campaign." ) );
             StringReplace( description, "%{spell}", Spell( _subType ).GetName() );
             return description;
         }
         case CampaignAwardData::TYPE_HIREABLE_HERO: {
-            std::string description( _( "%{hero} can be hired in the scenario." ) );
+            std::string description( _( "%{hero} can be hired during scenarios." ) );
             StringReplace( description, "%{hero}", Heroes( _subType, 0 ).GetName() );
             return description;
         }
         case CampaignAwardData::TYPE_DEFEAT_ENEMY_HERO: {
-            std::string description( _( "%{hero} has been defeated and will not appear in the subsequent scenarios." ) );
+            std::string description( _( "%{hero} has been defeated and will not appear in subsequent scenarios." ) );
             StringReplace( description, "%{hero}", Heroes( _subType, 0 ).GetName() );
             return description;
         }
