@@ -26,7 +26,10 @@
 #include <cstddef>
 #include <cstdint>
 #include <iterator>
+#include <map>
+#include <memory>
 #include <optional>
+#include <set>
 #include <string>
 #include <utility>
 #include <vector>
@@ -40,12 +43,15 @@
 #include "color.h"
 #include "cursor.h"
 #include "dialog.h"
+#include "dialog_selectitems.h"
+#include "editor_spell_selection.h"
 #include "editor_ui_helper.h"
 #include "game_hotkeys.h"
 #include "game_language.h"
 #include "icn.h"
 #include "image.h"
 #include "localevent.h"
+#include "mageguild.h"
 #include "map_format_helper.h"
 #include "map_format_info.h"
 #include "math_base.h"
@@ -53,16 +59,23 @@
 #include "race.h"
 #include "screen.h"
 #include "settings.h"
+#include "spell.h"
+#include "spell_storage.h"
 #include "statusbar.h"
 #include "translations.h"
 #include "ui_button.h"
 #include "ui_dialog.h"
+#include "ui_mage_guild.h"
 #include "ui_text.h"
 #include "ui_tool.h"
 #include "ui_window.h"
 
 namespace
 {
+    // The status bar has decorations on both sides.
+    // It is important not to render text over them.
+    constexpr int32_t statusBarDecorationsWidth{ 16 };
+
     class BuildingData
     {
     public:
@@ -300,6 +313,295 @@ namespace
 
         return isTown ? _( "Random Town Name" ) : _( "Random Castle Name" );
     }
+
+    bool mageGuildSpellsDialog( std::map<uint8_t, int32_t> & mustHaveSpells, std::vector<int32_t> & bannedSpells, const int race, const fheroes2::Rect & dialogRoi )
+    {
+        const CursorRestorer cursorRestorer( true, Cursor::POINTER );
+
+        fheroes2::Display & display = fheroes2::Display::instance();
+
+        fheroes2::ImageRestorer backgroundRestorer( display, dialogRoi.x, dialogRoi.y, dialogRoi.width, dialogRoi.height );
+
+        const bool isEvilInterface = Settings::Get().isEvilInterfaceEnabled();
+        const fheroes2::Sprite & backgroundSprite = fheroes2::AGG::GetICN( ( isEvilInterface ? ICN::STONEBAK_EVIL : ICN::STONEBAK ), 0 );
+        const fheroes2::Sprite & statusBarSprite = fheroes2::AGG::GetICN( ICN::CASLBAR, 0 );
+        const int32_t statusBarHeight = statusBarSprite.height();
+
+        fheroes2::Copy( backgroundSprite, 0, 0, display, dialogRoi.x, dialogRoi.y, dialogRoi.width, dialogRoi.height - statusBarHeight );
+
+        const bool hasLibraryCapability = ( race == Race::WZRD ) || ( race == Race::RAND );
+
+        // Guild image.
+        fheroes2::renderMageGuildBuilding( race, 5, dialogRoi.getPosition() );
+
+        // Show spells.
+        std::array<std::unique_ptr<fheroes2::SpellsInOneRow>, 5> spellRows;
+        for ( int32_t levelIndex = 0; levelIndex < static_cast<int32_t>( spellRows.size() ); ++levelIndex ) {
+            // REMEMBER! The level of spells as a parameter is in [1, 5] interval, but as an index is in [0, 4] interval.
+            const int32_t spellsCount = MageGuild::getMaxSpellsCount( levelIndex + 1, hasLibraryCapability );
+
+            SpellStorage spells;
+            spells.reserve( spellsCount );
+
+            for ( int32_t i = 0; i < spellsCount; ++i ) {
+                auto iter = mustHaveSpells.find( static_cast<uint8_t>( levelIndex * 10 + i ) );
+                if ( iter != mustHaveSpells.end() ) {
+                    spells.emplace_back( iter->second );
+                }
+                else {
+                    spells.emplace_back( Spell::RANDOM + levelIndex + 1 );
+                }
+            }
+
+            spellRows[levelIndex] = std::make_unique<fheroes2::SpellsInOneRow>( std::move( spells ) );
+
+            spellRows[levelIndex]->setPosition( { dialogRoi.x + 250, dialogRoi.y + 365 - 90 * levelIndex } );
+            spellRows[levelIndex]->redraw( display );
+        }
+
+        // OKAY button.
+        const fheroes2::Point statusBarOffset{ dialogRoi.x, dialogRoi.y + dialogRoi.height - statusBarHeight };
+
+        fheroes2::Button buttonOkay( statusBarOffset.x, statusBarOffset.y, ICN::BUTTON_OKAY_TOWN, 0, 1 );
+        buttonOkay.draw();
+
+        // EXIT button.
+        const int32_t buttonExitWidth = fheroes2::AGG::GetICN( ICN::BUTTON_GUILDWELL_EXIT, 0 ).width();
+        fheroes2::Button buttonExit( statusBarOffset.x + dialogRoi.width - buttonExitWidth, statusBarOffset.y, ICN::BUTTON_GUILDWELL_EXIT, 0, 1 );
+        buttonExit.draw();
+
+        fheroes2::ButtonSprite buttonBannedSpells;
+        {
+            const char * translatedText = fheroes2::getSupportedText( gettext_noop( "BANNED SPELLS" ), fheroes2::FontType::buttonReleasedWhite() );
+
+            fheroes2::Sprite released;
+            fheroes2::Sprite pressed;
+
+            getTextAdaptedSprite( released, pressed, translatedText, isEvilInterface ? ICN::EMPTY_EVIL_BUTTON : ICN::EMPTY_GOOD_BUTTON,
+                                  isEvilInterface ? ICN::STONEBAK_EVIL : ICN::STONEBAK );
+
+            buttonBannedSpells.setSprite( released, pressed );
+
+            buttonBannedSpells.setPosition( dialogRoi.x + dialogRoi.width - 10 - released.width(), dialogRoi.y + 10 );
+            addGradientShadow( released, display, buttonBannedSpells.area().getPosition(), { -5, 5 } );
+            buttonBannedSpells.draw();
+        }
+
+        // The original status bar image is much longer.
+        // Since we are adding 2 buttons on each side we have to copy only left and right parts of the bar.
+        const int32_t buttonOkayWidth = fheroes2::AGG::GetICN( ICN::BUTTON_OKAY_TOWN, 0 ).width();
+        const int32_t statusBarWidth = dialogRoi.width - buttonExitWidth - buttonOkayWidth;
+        fheroes2::Copy( statusBarSprite, 0, 0, display, statusBarOffset.x + buttonOkayWidth, statusBarOffset.y, statusBarWidth / 2, statusBarHeight );
+        const int32_t statusBarSecondPart = statusBarWidth - statusBarWidth / 2;
+        fheroes2::Copy( statusBarSprite, statusBarSprite.width() - statusBarSecondPart, 0, display, statusBarOffset.x + buttonOkayWidth + statusBarWidth / 2,
+                        statusBarOffset.y, statusBarSecondPart, statusBarHeight );
+
+        StatusBar statusBar;
+        statusBar.setRoi( { dialogRoi.x + buttonOkayWidth + statusBarDecorationsWidth, statusBarOffset.y, statusBarWidth - 2 * statusBarDecorationsWidth, 0 } );
+
+        display.render( dialogRoi );
+
+        std::vector<int32_t> currentBannedSpells = bannedSpells;
+
+        auto spellSelectProcessing = [&spellRows, &dialogRoi, &currentBannedSpells, &display]( const size_t levelIndex ) {
+            // Exclude the already selected spells and the banned spells.
+            const int32_t spellLevel = static_cast<int32_t>( levelIndex ) + 1;
+
+            const int32_t currentSpellId = spellRows[levelIndex]->getCurrentSpell().GetID();
+
+            std::set<int32_t> spellsToExclude;
+            for ( const Spell spell : spellRows[levelIndex]->getSpells() ) {
+                const int32_t spellId = spell.GetID();
+                // Do not exclude currently selected spell from the list.
+                if ( spellId != currentSpellId || ( spellId >= Spell::RANDOM && spellId <= Spell ::RANDOM5 ) ) {
+                    spellsToExclude.insert( spellId );
+                }
+            }
+            for ( const int32_t spellId : currentBannedSpells ) {
+                if ( Spell( spellId ).Level() == spellLevel ) {
+                    spellsToExclude.insert( spellId );
+                }
+            }
+
+            const Spell spell = Dialog::selectSpell( spellRows[levelIndex]->getCurrentSpell().GetID(), true, spellsToExclude, spellLevel );
+
+            if ( spell != Spell::NONE ) {
+                spellRows[levelIndex]->setCurrentSpell( spell );
+                spellRows[levelIndex]->redrawCurrentSpell( display );
+            }
+
+            display.render( dialogRoi );
+        };
+
+        LocalEvent & eventHandler = LocalEvent::Get();
+
+        bool hasChanges = false;
+        int32_t currentBannedSpellsLevel = 1;
+
+        std::string statusBarMessage;
+
+        while ( eventHandler.HandleEvents() ) {
+            if ( eventHandler.MouseClickLeft( buttonExit.area() ) || Game::HotKeyPressEvent( Game::HotKeyEvent::DEFAULT_CANCEL ) ) {
+                break;
+            }
+
+            if ( eventHandler.MouseClickLeft( buttonOkay.area() ) || Game::HotKeyPressEvent( Game::HotKeyEvent::DEFAULT_OKAY ) ) {
+                // Update must have spells.
+                for ( size_t levelIndex = 0; levelIndex < spellRows.size(); ++levelIndex ) {
+                    auto spells = spellRows[levelIndex]->getSpells();
+                    for ( size_t position = 0; position < spells.size(); ++position ) {
+                        const uint8_t key = static_cast<uint8_t>( levelIndex * 10 + position );
+                        auto iter = mustHaveSpells.find( key );
+
+                        // Random and invalid spells have level 0.
+                        if ( spells[position].Level() != static_cast<int32_t>( levelIndex ) + 1 ) {
+                            if ( iter != mustHaveSpells.end() ) {
+                                mustHaveSpells.erase( iter );
+                                hasChanges = true;
+                            }
+                        }
+                        else {
+                            const int32_t spellId = spells[position].GetID();
+
+                            if ( iter != mustHaveSpells.end() ) {
+                                if ( spellId != iter->second ) {
+                                    iter->second = spellId;
+                                    hasChanges = true;
+                                }
+                            }
+                            else {
+                                mustHaveSpells[key] = spellId;
+                                hasChanges = true;
+                            }
+                        }
+                    }
+                }
+
+                bannedSpells = std::move( currentBannedSpells );
+
+                break;
+            }
+
+            if ( eventHandler.MouseClickLeft( buttonBannedSpells.area() ) ) {
+                std::array<std::vector<int32_t>, 5> bannedSpellsContainer;
+
+                for ( const int32_t spellId : currentBannedSpells ) {
+                    const int32_t level = Spell( spellId ).Level();
+                    assert( level >= 1 && level <= 5 );
+
+                    bannedSpellsContainer[level - 1].push_back( spellId );
+                }
+
+                auto getMageGuildTitle = []( const int level ) {
+                    switch ( level ) {
+                    case 1:
+                        return fheroes2::getBuildingName( 0, BUILD_MAGEGUILD1 );
+                    case 2:
+                        return fheroes2::getBuildingName( 0, BUILD_MAGEGUILD2 );
+                    case 3:
+                        return fheroes2::getBuildingName( 0, BUILD_MAGEGUILD3 );
+                    case 4:
+                        return fheroes2::getBuildingName( 0, BUILD_MAGEGUILD4 );
+                    case 5:
+                        return fheroes2::getBuildingName( 0, BUILD_MAGEGUILD5 );
+                    default:
+                        assert( 0 );
+                        return "";
+                    }
+                };
+
+                int32_t selectedLevel = currentBannedSpellsLevel;
+                while ( Editor::openSpellSelectionWindow( getMageGuildTitle( selectedLevel ), selectedLevel, bannedSpellsContainer[selectedLevel - 1], true,
+                                                          MageGuild::getMaxSpellsCount( selectedLevel, hasLibraryCapability ), true ) ) {
+                    if ( selectedLevel == currentBannedSpellsLevel ) {
+                        // The banned spells dialog was closed with confirmation of changes.
+                        std::vector<int32_t> result( std::move( bannedSpellsContainer[0] ) );
+                        result.reserve( result.size() + bannedSpellsContainer[1].size() + bannedSpellsContainer[2].size() + bannedSpellsContainer[3].size()
+                                        + bannedSpellsContainer[4].size() );
+                        result.insert( result.end(), bannedSpellsContainer[1].begin(), bannedSpellsContainer[1].end() );
+                        result.insert( result.end(), bannedSpellsContainer[2].begin(), bannedSpellsContainer[2].end() );
+                        result.insert( result.end(), bannedSpellsContainer[3].begin(), bannedSpellsContainer[3].end() );
+                        result.insert( result.end(), bannedSpellsContainer[4].begin(), bannedSpellsContainer[4].end() );
+
+                        if ( result != currentBannedSpells ) {
+                            currentBannedSpells = std::move( result );
+
+                            // Remove the banned spells from the spell Rows.
+                            for ( const int32_t spellId : currentBannedSpells ) {
+                                const Spell spell( spellId );
+                                const int32_t spellLevel = spell.Level();
+
+                                // Banned spells must be always valid ones.
+                                assert( spellLevel >= 1 && spellLevel <= 5 );
+
+                                auto & spellRow = spellRows[spellLevel - 1];
+
+                                if ( spellRow->checkSpellAndMakeItCurrent( spell ) ) {
+                                    spellRow->setCurrentSpell( Spell::RANDOM + spellLevel );
+                                    spellRow->redrawCurrentSpell( display );
+                                }
+                            }
+
+                            hasChanges = true;
+                        }
+
+                        break;
+                    }
+
+                    currentBannedSpellsLevel = selectedLevel;
+                }
+            }
+
+            if ( eventHandler.isMouseCursorPosInArea( buttonOkay.area() ) ) {
+                statusBarMessage = _( "Click to accept the changes made." );
+
+                if ( eventHandler.isMouseRightButtonPressedInArea( buttonOkay.area() ) ) {
+                    fheroes2::showStandardTextMessage( _( "Okay" ), statusBarMessage, Dialog::ZERO );
+                }
+            }
+            else if ( eventHandler.isMouseCursorPosInArea( buttonExit.area() ) ) {
+                statusBarMessage = _( "Exit this dialog, discarding the changes made." );
+
+                if ( eventHandler.isMouseRightButtonPressedInArea( buttonExit.area() ) ) {
+                    fheroes2::showStandardTextMessage( _( "Exit" ), statusBarMessage, Dialog::ZERO );
+                }
+            }
+            else if ( eventHandler.isMouseCursorPosInArea( buttonBannedSpells.area() ) ) {
+                statusBarMessage = _( "Click to ban spells from appearing in the Mage Guild." );
+
+                if ( eventHandler.isMouseRightButtonPressedInArea( buttonBannedSpells.area() ) ) {
+                    fheroes2::showStandardTextMessage( _( "Banned Spells" ), statusBarMessage, Dialog::ZERO );
+                }
+            }
+
+            buttonExit.drawOnState( eventHandler.isMouseLeftButtonPressedAndHeldInArea( buttonExit.area() ) );
+            buttonOkay.drawOnState( eventHandler.isMouseLeftButtonPressedAndHeldInArea( buttonOkay.area() ) );
+            buttonBannedSpells.drawOnState( eventHandler.isMouseLeftButtonPressedAndHeldInArea( buttonBannedSpells.area() ) );
+
+            for ( size_t levelIndex = 0; levelIndex < 5; ++levelIndex ) {
+                if ( spellRows[levelIndex]->queueEventProcessing( true ) ) {
+                    if ( eventHandler.MouseClickLeft() ) {
+                        spellSelectProcessing( levelIndex );
+                    }
+
+                    break;
+                }
+            }
+
+            if ( statusBarMessage.empty() ) {
+                statusBar.ShowMessage( _( "Select spells that will appear in the Mage Guild." ) );
+            }
+            else {
+                statusBar.ShowMessage( std::move( statusBarMessage ) );
+                statusBarMessage.clear();
+            }
+        }
+
+        backgroundRestorer.restore();
+        display.render( dialogRoi );
+
+        return hasChanges;
+    }
 }
 
 namespace Editor
@@ -348,6 +650,13 @@ namespace Editor
         };
         drawCastleName();
 
+        // Button to open dialog to set forced and disabled spells in the Mages Guild.
+        fheroes2::ButtonSprite buttonSpells;
+        {
+            const char * translatedText = fheroes2::getSupportedText( gettext_noop( "SET SPELLS" ), fheroes2::FontType::buttonReleasedWhite() );
+            window.renderTextAdaptedButtonSprite( buttonSpells, translatedText, { 219, 78 }, fheroes2::StandardWindow::Padding::TOP_CENTER );
+        }
+
         // Allow castle building checkbox.
         fheroes2::Point dstPt( dialogRoi.x + rightPartOffsetX + 10, dialogRoi.y + 130 );
         fheroes2::MovableSprite allowCastleSign;
@@ -485,10 +794,7 @@ namespace Editor
                         statusBarOffset.y, statusBarSecondPart, statusBarHeight );
 
         StatusBar statusBar;
-        // The status bar has decorations on both sides.
-        // It is important not to render text over them.
-        constexpr int32_t decorationsWidth{ 16 };
-        statusBar.setRoi( { dialogRoi.x + buttonOkayWidth + decorationsWidth, statusBarOffset.y, statusBarWidth - 2 * decorationsWidth, 0 } );
+        statusBar.setRoi( { dialogRoi.x + buttonOkayWidth + statusBarDecorationsWidth, statusBarOffset.y, statusBarWidth - 2 * statusBarDecorationsWidth, 0 } );
 
         display.render( dialogWithShadowRoi );
 
@@ -502,6 +808,7 @@ namespace Editor
             buttonExit.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonExit.area() ) );
             buttonResetCastleName.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonResetCastleName.area() ) );
             buttonResetArmy.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonResetArmy.area() ) );
+            buttonSpells.drawOnState( le.isMouseLeftButtonPressedAndHeldInArea( buttonSpells.area() ) );
 
             if ( le.MouseClickLeft( buttonOkay.area() ) || Game::HotKeyPressEvent( Game::HotKeyEvent::DEFAULT_OKAY ) ) {
                 break;
@@ -514,6 +821,9 @@ namespace Editor
             if ( le.MouseClickLeft( buttonRestrictBuilding.area() ) ) {
                 buildingRestriction = !buildingRestriction;
             }
+            else if ( le.MouseClickLeft( buttonSpells.area() ) ) {
+                mageGuildSpellsDialog( castleMetadata.mustHaveSpells, castleMetadata.bannedSpells, race, dialogRoi );
+            }
 
             buttonRestrictBuilding.drawOnState( buildingRestriction || le.isMouseLeftButtonPressedAndHeldInArea( buttonRestrictBuilding.area() ) );
 
@@ -646,6 +956,13 @@ namespace Editor
                     message = _( "Set custom Castle Army." );
                 }
             }
+            else if ( le.isMouseCursorPosInArea( buttonSpells.area() ) ) {
+                message = _( "Set spells to appear in the Mage Guild." );
+
+                if ( le.isMouseRightButtonPressedInArea( buttonSpells.area() ) ) {
+                    fheroes2::showStandardTextMessage( _( "Set Spells" ), message, Dialog::ZERO );
+                }
+            }
             else if ( le.isMouseCursorPosInArea( buttonOkay.area() ) ) {
                 message = _( "Click to accept the changes made." );
 
