@@ -48,7 +48,7 @@ namespace Game
 
 #if defined( WITH_DEBUG )
         // This hotkey is only for debug mode.
-        DEBUG_TOGGLE_DEVELOPER_MODE,
+        GLOBAL_TOGGLE_DEVELOPER_MODE,
 #endif
 
         MAIN_MENU_NEW_GAME,
@@ -146,6 +146,7 @@ namespace Game
         BATTLE_OPTIONS,
         BATTLE_SKIP,
         BATTLE_CAST_SPELL,
+        BATTLE_TOGGLE_TURN_ORDER_DISPLAY,
 
         TOWN_DWELLING_LEVEL_1,
         TOWN_DWELLING_LEVEL_2,
