@@ -28,6 +28,7 @@
 #include <cstddef>
 #include <cstdint>
 #include <functional>
+#include <memory>
 #include <ostream>
 #include <string>
 #include <utility>
@@ -133,43 +134,63 @@ namespace
 
     void ShowNewWeekDialog()
     {
-        // restore the original music on exit
+        // Restore the original music on exit
         const AudioManager::MusicRestorer musicRestorer;
 
         const bool isNewMonth = world.BeginMonth();
 
         AudioManager::PlayMusic( isNewMonth ? MUS::NEW_MONTH : MUS::NEW_WEEK, Music::PlaybackMode::PLAY_ONCE );
 
+        auto [headerText, messageText] = [isNewMonth]() -> std::pair<std::string, std::string> {
+            if ( isNewMonth ) {
+                return { _( "New Month!" ), _( "Astrologers proclaim the Month of the %{name}." ) };
+            }
+
+            return { _( "New Week!" ), _( "Astrologers proclaim the Week of the %{name}." ) };
+        }();
+
         const Week & week = world.GetWeekType();
 
-        // head
-        std::string message = isNewMonth ? _( "Astrologers proclaim the Month of the %{name}." ) : _( "Astrologers proclaim the Week of the %{name}." );
-        StringReplace( message, "%{name}", week.GetName() );
-        message += "\n\n";
+        StringReplace( messageText, "%{name}", week.GetName() );
+        messageText += "\n\n";
 
-        if ( week.GetType() == WeekName::MONSTERS ) {
+        std::unique_ptr<const fheroes2::MonsterDialogElement> monsterDialogElement;
+
+        switch ( week.GetType() ) {
+        case WeekName::MONSTERS: {
             const Monster monster( week.GetMonster() );
-            const uint32_t count = isNewMonth ? Castle::GetGrownMonthOf() : Castle::GetGrownWeekOf();
+            assert( monster.isValid() );
 
-            if ( monster.isValid() && count ) {
-                if ( isNewMonth )
-                    message += 100 == Castle::GetGrownMonthOf() ? _( "After regular growth, the population of %{monster} is doubled!" )
-                                                                : _n( "After regular growth, the population of %{monster} increases by %{count} percent!",
-                                                                      "After regular growth, the population of %{monster} increases by %{count} percent!", count );
-                else
-                    message += _( "%{monster} growth +%{count}." );
-                StringReplaceWithLowercase( message, "%{monster}", monster.GetMultiName() );
-                StringReplace( message, "%{count}", count );
-                message += "\n\n";
+            const uint32_t count = isNewMonth ? Castle::GetGrownMonthOf() : Castle::GetGrownWeekOf();
+            assert( count > 0 );
+
+            if ( isNewMonth ) {
+                messageText += ( count == 100 ) ? _( "After regular growth, the population of %{monster} is doubled!" )
+                                                : _n( "After regular growth, the population of %{monster} increases by %{count} percent!",
+                                                      "After regular growth, the population of %{monster} increases by %{count} percent!", count );
             }
+            else {
+                messageText += _( "%{monster} growth +%{count}." );
+            }
+
+            StringReplaceWithLowercase( messageText, "%{monster}", monster.GetMultiName() );
+            StringReplace( messageText, "%{count}", count );
+
+            monsterDialogElement = std::make_unique<const fheroes2::MonsterDialogElement>( monster );
+
+            break;
+        }
+        case WeekName::PLAGUE:
+            messageText += _( "All populations are halved." );
+            break;
+        default:
+            messageText += _( "All dwellings increase population." );
+            break;
         }
 
-        if ( week.GetType() == WeekName::PLAGUE )
-            message += _( " All populations are halved." );
-        else
-            message += _( " All dwellings increase population." );
-
-        fheroes2::showStandardTextMessage( isNewMonth ? _( "New Month!" ) : _( "New Week!" ), message, Dialog::OK );
+        fheroes2::showStandardTextMessage( std::move( headerText ), std::move( messageText ), Dialog::OK,
+                                           monsterDialogElement ? std::vector<const fheroes2::DialogElement *>{ monsterDialogElement.get() }
+                                                                : std::vector<const fheroes2::DialogElement *>{} );
     }
 
     void ShowWarningLostTownsDialog()
